//
//  MLMorphem.h
//  bmcc
//
//  Created by Matthias Lange on Tue May 18 2004.
//  Copyright (c) 2004 Benjamin Dittes & Matthias Lange. All rights reserved.
//
/*!
	@header MLMorphem.h
	@abstract This class provides an abstraction of a morphem needed for the lexical analysis.
	@discussion This class a is model class which is needed by the lexical analysis to store its information. This class is KVC compliant.
 */

#import <Foundation/Foundation.h>

@interface MLMorphem : NSObject
{

	int _line, _column;
	NSString *_semantik, *_syntax;
	NSRange _range;
}

/*  ================
	accessor methods
	================ 
*/
/*!
	@method		- (NSString *)semantik
	@abstract   returns the semantic of the morphem
	@discussion This method returns a string containing the semantic of the morphem.
 */
- (NSString *)semantic;

/*!
	@method		- (NSString *)syntax
	@abstract   returns the syntax of the morphem
	@discussion This method returns a string containing the syntax of the morphem.
*/
- (NSString *)syntax;

/*!
	@method		- (int)line
	@abstract   returns the line number of the morphem
	@discussion This method returns an int containing the line number of the morphem.
*/
- (int)line;

/*!
	@method		- (int)column
	@abstract   returns the column number of the morphem
	@discussion This method returns an int containing the column number of the morphem.
*/
- (int)column;

/*!
	@method		- (NSRange)range
	@abstract   returns the range of the morphem
	@discussion This method returns a NSRange containing the range (length and starting position in source code) of the morphem. It is used to highlight a selected morphem in the source code.
*/
- (NSRange)range;

/*  ===============
	setting methods
	===============
*/
/*!
	@method		- (void)setSemantic:(NSString *)sem
	@abstract   set the semantic of the morphem
	@discussion This method is used to set the semantic of the morphem.
*/
- (void)setSemantic:(NSString *)sem;

/*!
	@method		- (void)setSyntax: (NSString *)syn
	@abstract   set the syntax of the morphem
	@discussion This method is used to set the syntax of the morphem.
*/
- (void)setSyntax: (NSString *)syn;

/*!
	@method		- (void)setLine: (int)l;
	@abstract   set the line of the morphem
	@discussion This method is used to set the line of the morphem.
*/
- (void)setLine: (int)l;

/*!
	@method		- (void)setColumn: (int)col
	@abstract   set the column of the morphem
	@discussion This method is used to set the column of the morphem.
*/
- (void)setColumn: (int)col;

/*!
	@method		- (void)setRange:(NSRange)r
	@abstract   set the range of the morphem
	@discussion This method is used to set the range of the morphem. The range is the length and the position in source code of the morphem.
*/
- (void)setRange:(NSRange)r;

/*!
	@method		- (BOOL)show
	@abstract   returns always YES
	@discussion This method returns always YES and is used to determine when the morphem is selected and to activate user interface elements.
*/
- (BOOL)show;

@end
