//
//  STreeNode.m
//  bmcc
//
//  Created by Benjamin Dittes
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#include "STreeNode.h"

@implementation STreeNode

// default constructor
- (id)initWithParent:(STreeNode *)myParent andName: (NSString *)name
{
	[super init];
	_parent = myParent;
	_name = name;
	_entries = [[NSMutableDictionary alloc] init];
	_children = [[NSMutableDictionary alloc] init];
	_lineNumber = 0;
	return self;
}

// accessors

- (NSString *)name
{
	return _name;
}
- (STreeNode *)parent
{
	return _parent;
}
- (NSMutableDictionary *)entries
{
	return _entries;
}
- (NSMutableDictionary *)children
{
	return _children;
}
- (int) lineNumber
{
	return _lineNumber;
}
- (void) setLineNumber: (int)newNumber
{
	_lineNumber = newNumber;
}
- (BOOL) isNameDefined: (NSString *)name withTypes: (NSString *)typeStr
{
	NSString *type = [_entries objectForKey: name];
	if(type!=nil)
	{

		NSArray *parts = [typeStr componentsSeparatedByString:@":"];
		NSEnumerator *pE = [parts objectEnumerator];
		NSString *part;
		while(part = [pE nextObject])
		{
			if([part isEqualToString: type])
				return YES;
		}
	}
	if(_parent == nil)
		return NO;
	return [_parent isNameDefined: name withTypes: typeStr];
}
- (BOOL) defineName: (NSString *)name withType: (NSString *)type
{
	NSString *t = [_entries objectForKey: name];
	if(t != nil)
		return NO;
	[_entries setObject: type forKey: name];
	return YES;
}
- (STreeNode *) createChildNode: (NSString *)name
{
	STreeNode *child = [_children objectForKey: name];
	if(child!=nil)
		return nil;
	child = [[STreeNode alloc] initWithParent: self andName: name];
	[_children setObject: child forKey: name];
	return child;
}

- (STreeNode *) getChildNode: (NSString *)name
{
	return [_children objectForKey: name];
}

- (STreeNode *) findNode: (NSString *)name
{
	if([_name isEqualToString: name])
		return self;
	STreeNode *E = [_children objectForKey: name];
	if(E!=nil)
		return E;
	if(_parent==nil)
		return nil;
	return [_parent findNode: name];
}

- (NSMutableArray *) getDefinitionsWithType: (NSString *) type
{
	NSMutableArray *Erg = [[NSMutableArray alloc] init];
	NSEnumerator *kE = [_entries keyEnumerator];
	NSString *name;
	while(name = [kE nextObject])
	{
		if([[_entries objectForKey: name] isEqualToString: type])
			[Erg addObject: name];
	}
	return Erg;
}

- (void)dealloc
{
    [_children release];
    [_entries release];
    [super dealloc];	
}
@end