//
//  bmccMain.h
//  bmcc
//
//  Created by Matthias Lange on Sun May 09 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

@class editorToolbarController;
@class errorDrawerController;
@class GrammarController;
@class MLLexController;
@class lexModel;
@class grammarModel;
@class MLGrammarModel;
@class lexAnalyse;
@class MLMorphemPanelController;
@class MLMorphem;
@class MLAnalyseTableController;
@class MLIOController;
@class JSDTextView;

//#define _DEBUGMAIN

@interface bmccMain : NSObject
{
	IBOutlet NSWindow *editorWindow, *machineCodeWindow;
	IBOutlet JSDTextView *editorTextView;
	IBOutlet NSTextView *errorTextView;
	IBOutlet NSTextView *passZeroTextView;
	IBOutlet NSTextView *passOneTextView;
	IBOutlet NSTextView *passTwoTextView;
	IBOutlet NSTextView *completeTextView;
	IBOutlet NSMenuItem *showErrorDrawerMenu;
	IBOutlet NSDrawer *errorDrawer;
//	IBOutlet NSScrollView *scrollView;
	NSToolbar *toolbar;
	
	editorToolbarController *myEditorToolbarController;
	NSOpenPanel *openPanel;
	NSSavePanel *savePanel;
	NSString *currentDocumentPath;
	
	NSArray *sourceFileTypes;
	NSArray *xmlFileTypes;
	
	// Controllers
	GrammarController *myGrammarController;
	MLLexController *myLexController;
	IBOutlet MLMorphemPanelController *myMorphemPanelController;
	MLAnalyseTableController *analyseTableController;
	MLIOController *ioController;
	
	lexAnalyse *lex;
	
	// Models
	MLGrammarModel *mlGrammar;
	grammarModel *grammar;
	lexModel *lexspec;
	
	// array with the morphems
	NSMutableArray *morpheme;
	NSMutableArray *morphems;
	NSMutableArray *syntaxTree;
	NSArray *_analyseTable;
	
	BOOL syntaxAnalysis, lexAnalysis, somethingChanged, grammarLoaded;
	BOOL codePassOneDone, codePassTwoDone, codePassZeroDone, codeCompleteDone;
	
	NSRange theEnd;
}

// File menu actions
- (IBAction)openSourceFile:(id)sender;
- (IBAction)saveDocument:(id)sender;
- (IBAction)saveDocumentAs:(id)sender;
- (IBAction)openGrammarFile:(id)sender;

// View menu actions
- (IBAction)showHideErrorDrawer:(id)sender;
- (IBAction)showGrammar:(id)sender;
- (IBAction)showAnalyseTable:(id)sender;
//- (IBAction)showMorphemPanel:(id)sender;
- (void)showErrorDrawer;

// Option menu actions
- (IBAction)createCodePassZero:(id)sender;
- (IBAction)createCodeComplete:(id)sender;
- (IBAction)createCodePassOne:(id)sender;
- (IBAction)createCodePassTwo:(id)sender;

// io methods
- (void)saveDocumentAs;

// toolbar Actions
- (IBAction)customize:(id)sender;
- (IBAction)showhide:(id)sender;

// methods for lexical analysis
- (IBAction)lexicalAnalysis:(id)sender;
- (void)lexicalAnalysis;

// methods for syntax analysis
- (IBAction)syntaxAnalysis:(id)sender;
- (void)syntaxAnalysis;

// method for displaying morphems in the source code
- (void)showMorphem:(MLMorphem *)morphem;

// returns the current source code displayed in the editor
- (NSString *)sourceCode;

- (IBAction)saveMachineCode:(id)sender;

- (NSArray *)analyseTable;

- (MLGrammarModel *)mlGrammarModel;
- (grammarModel *)grammarModel;
- (lexModel *)lexModel;
// returns the actual morphem panel controller
- (MLMorphemPanelController *)morphemPanelController;
- (BOOL)grammarLoaded;
- (void)setLexicalAnalysisStatus: (BOOL)status;

// displays errors in the error drawer
- (void)showErrors:(NSString *)errorText;

- (NSString *)createCodeForPass:(NSMutableArray *)semantikCode;

@end
