#import "grammarController.h"
#import "grammarModel.h"
#import "MLGrammarModel.h"
#import "MLGrammarRule.h"
#import "MLRuleOption.h"
#import "MLTerminal.h"
#import "MLNonTerminal.h"
#import "bmccMain.h"
#import "optionModel.h"

@implementation GrammarController

- (id) init
{
    [super init];

	currentRuleName = [[NSString alloc] init];
	currentOptions = [[NSMutableArray alloc] init];
	tmpterminals = [[NSMutableArray alloc] initWithCapacity: 1];
	tmpNonTerminals = [[NSMutableArray alloc] initWithCapacity: 1];
	// set up the array controllers
	terminalArrayController = [[NSArrayController alloc] init];
	[terminalArrayController setObjectClass: [MLTerminal class]];
	[terminalArrayController setSelectsInsertedObjects: NO];
	nonTerminalArrayController = [[NSArrayController alloc] init];
	[nonTerminalArrayController setObjectClass: [MLNonTerminal class]];
	[nonTerminalArrayController setSelectsInsertedObjects: NO];
	nterminal = NO;
	terminal = NO;
	rule = NO;
	option = NO;
	syntax = NO;
	semantik = NO;
	isSetup = NO;
	ruleNameCount = 0;
    return self;
}

- (id)initWithParent:(bmccMain *)parent
{
	[self init];
	myParent = parent;
	myGrammarModel = [myParent mlGrammarModel];
	grammar = [myParent grammarModel];
    return self;
}

- (void)showGrammar
{
	if(!grammarWindow)
	{
		if (![NSBundle loadNibNamed:@"Grammar" owner:self])
		{
			NSLog(@"Failed to load Grammar.nib");
			NSBeep();
			return;
		}
	}
	[grammarWindow makeKeyAndOrderFront:nil];
	// set data source for grammar outline view
	[grammarOutlineView setDataSource: self];
	[objectController addObject: myGrammarModel];
	[terminalTableColumn bind: @"value"
		toObject: terminalArrayController
	 withKeyPath: @"arrangedObjects.name"
		 options: nil];
	[nonTerminalTableColumn bind: @"value"
					 toObject: nonTerminalArrayController
				  withKeyPath: @"arrangedObjects.name"
					  options: nil];
	if(![myParent grammarLoaded])
	{
		NSString *noGrammarLoaded;
		noGrammarLoaded = NSLocalizedString(@"No grammar loaded", @"This is the window title of the grammar window when no grammar is loaded");
		[grammarWindow setTitle: noGrammarLoaded];
		[terminalCountTextField setStringValue: @""];
		[nonTerminalCountTextField setStringValue: @""];
	}
	else
	{
		// fill the outline view with the rules
		[self fillOutlineView];
	}
}

- (void)fillOutlineView
{
	if([grammarWindow isVisible])
	{
		NSArray *rules = [myGrammarModel rules];
		int i, count = [rules count];
		for(i = 0; i < count; i++)
		{
			MLGrammarRule *n = [grammarOutlineView itemAtRow: i];
			[n setName: [[rules objectAtIndex: i] name]];
		}
		[grammarOutlineView reloadData];
	}
}

- (void)parseGrammarFromData:(NSData *)data inDirectory: (NSString *)dir
{
	NSXMLParser *myXmlParser;
	
	workingDir = dir;
	myXmlParser = [[NSXMLParser alloc] initWithData: data];
	[myXmlParser setDelegate: self];
	if(!myXmlParser)
		NSLog(@"Error creating parser for grammar.");
	if([myXmlParser parse])
		NSLog(@"Grammar OK");
	else
		NSLog(@"Grammar not OK");
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
	if([elementName isEqualToString: @"grammar"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSString *grammarName = [attributeDict objectForKey:@"name"];
			NSString *epsSymbol = [attributeDict objectForKey:@"eps"];
			NSString *startSymbol = [attributeDict objectForKey: @"start"];
			NSString *lexFile = [attributeDict objectForKey: @"lexspec"];
			NSLog(@"Name: %@", grammarName);
			NSLog(@"Eps: %@", epsSymbol);
			NSLog(@"Start: %@", startSymbol);
			NSLog(@"Lex: %@", lexFile);
		#endif
		NSString *tmp = [[NSString alloc] initWithString: [workingDir stringByDeletingLastPathComponent]];
		tmp = [tmp stringByAppendingString: @"/"];
		// ======
		[myGrammarModel setName: [attributeDict objectForKey: @"name"]];
		[myGrammarModel setEpsilonSymbol: [attributeDict objectForKey: @"eps"]];
		[myGrammarModel setStartSymbol: [attributeDict objectForKey: @"start"]];
		[myGrammarModel setLexSpecification: [tmp stringByAppendingString: (NSString *)[attributeDict objectForKey: @"lexspec"]]];
		// ======
		[grammar setLexSpec: [tmp stringByAppendingString: (NSString *)[attributeDict objectForKey: @"lexspec"]]];
		[grammar setGrammarName: [attributeDict objectForKey: @"name"]];
		[grammar setEpsilonSymbol: [attributeDict objectForKey: @"eps"]];
		[grammar setStartSymbol: [attributeDict objectForKey: @"start"]];
	}
	if([elementName isEqualToString: @"nterminal"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Non terminal beginning found.");
		#endif
		nterminal = YES;
		tmpElementValue = [[NSMutableString alloc] init];
	}
	if([elementName isEqualToString: @"terminal"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Terminal found.");
		#endif
		terminal = YES;
		tmpElementValue = [[NSMutableString alloc] init];
	}
	if([elementName isEqualToString: @"rules"])
	{
		ruleCount = 0;
		grammarRules = [[NSMutableArray alloc] initWithCapacity: 1];
	}
	if([elementName isEqualToString: @"rule"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Rule beginning found.");
			//NSLog(ruleName);
		#endif
		rule = YES;
		NSString *ruleName = [attributeDict objectForKey: @"name"];
		// ======
		ruleNameCount++;
		grammarRule = [[MLGrammarRule alloc] init];
		[grammarRule setName: ruleName];
		options = [[NSMutableArray alloc] init];
		// ======
		[grammar addRuleToList: ruleName];
		currentRuleName = ruleName;
		currentOptions = [[NSMutableArray alloc] initWithCapacity: 1];
	}
	if([elementName isEqualToString: @"option"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Option beginning found.");
		#endif
		model = [[optionModel alloc] initWithLeftSide: currentRuleName];
		// ====
		ruleOption = [[MLRuleOption alloc] init];
		// add this option to the rules option list
		[options addObject: ruleOption];
		// increase the rule count
		ruleCount++;
		[ruleOption setNumber: ruleCount];
		// ====
		option = YES;
	}
	if([elementName isEqualToString: @"syntax"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Syntax beginning found.");
		#endif
		syntax = YES;
		tmpElementValue = [[NSMutableString alloc] init];
	}
	if([elementName isEqualToString: @"semantik"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Semantik beginning found.");
		#endif
		semantik = YES;
		tmpElementValue = [[NSMutableString alloc] init];
	}
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{
	if([elementName isEqualToString: @"nterminal"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Non terminal end found with non terminal: %@.", tmpElementValue);
		#endif
		[grammar addNTerminal: tmpElementValue];
		MLNonTerminal *nterm = [[MLNonTerminal alloc] init];
		[nterm setName: tmpElementValue];
		[nonTerminalArrayController addObject: nterm];
		[nterm retain];
		[tmpNonTerminals addObject: nterm];
		nterminal = NO;
	}
	if([elementName isEqualToString: @"nterminals"])
	{
		[myGrammarModel setNonTerminals: tmpNonTerminals];
		[myGrammarModel setNonTerminalCount: [tmpNonTerminals count]];
	}
	if([elementName isEqualToString: @"terminal"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Terminal end found with terminal: %@.", tmpElementValue);
		#endif
		[grammar addTerminal: tmpElementValue];
		MLTerminal *term = [[MLTerminal alloc] init];
		[term setName: tmpElementValue];
		[terminalArrayController addObject: term];
		[term retain];
		[tmpterminals addObject: term];
		terminal = NO;
	}
	if([elementName isEqualToString: @"terminals"])
	{
		[myGrammarModel setTerminals: tmpterminals];
		[myGrammarModel setTerminalCount: [tmpterminals count]];
	}
	if([elementName isEqualToString: @"rule"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Rule end found for rule %@", currentRuleName);
		#endif
		// ===
		// add the options to the rule
		[grammarRule setOptions: (NSArray *)options];
		// add the rule to the model
		[grammarRules addObject: grammarRule];
		// ===
		rule = NO;
		[grammar addRuleWithName: currentRuleName withOptions: currentOptions];
	}
	if([elementName isEqualToString: @"rules"])
	{
		// add the rule to the grammar model
		[myGrammarModel setRules: grammarRules];
	}
	if([elementName isEqualToString: @"option"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Option end found.");
		#endif
		option = NO;
		[model parse];
		[currentOptions addObject: model];
	}
	if([elementName isEqualToString: @"syntax"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Syntax end found with syntax: %@.", tmpElementValue);
		#endif
		// ===
		[ruleOption setSyntax: tmpElementValue];
		// ===
		[model addSyntax: tmpElementValue];
		syntax = NO;
	}
	if([elementName isEqualToString: @"semantik"])
	{
		#ifdef _DEBUGGRAMMARCONTROLLER
			NSLog(@"Semantik end found with semantik: %@.", tmpElementValue);
		#endif
		// ===
		[ruleOption setSemantik: tmpElementValue];
		// ===
		[model addSemantik: tmpElementValue];
		semantik = NO;
	}
	if([elementName isEqualToString: @"grammar"])
		[self fillOutlineView];
	
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
	#ifdef _DEBUGGRAMMARCONTROLLER
		NSLog(string);
	#endif
	// we have found a non terminal
	if(nterminal)
		tmpElementValue = [tmpElementValue stringByAppendingString: string];
	// we have found a terminal
	if(terminal)
		tmpElementValue = [tmpElementValue stringByAppendingString: string];
	// we have found a rules syntax
	if(rule && option && syntax)
		tmpElementValue = [tmpElementValue stringByAppendingString: string];
	// we have found a rules semantik
	if(rule && option && semantik)
		tmpElementValue = [tmpElementValue stringByAppendingString: string];
}

/*- (void)addNode:(NSString *)nodeName atRow:(int)row
{
   // int selectedRow = [grammarOutlineView i];
    MLGrammarRule *n = [grammarOutlineView itemAtRow: row];
	NSLog(@"Item at row: %i is %@", row, [n name]);
    MLGrammarRule *newChild= [[MLGrammarRule alloc] init];
	[newChild setName: nodeName];
	*/
 /*   if(row == -1)
	{
        [rootNode addChild: newChild];
	}
    else */
/*	[n addChild: newChild];
	
    [grammarOutlineView reloadData];
	[newChild release];
}*/

- (id)outlineView:(NSOutlineView *)ov child:(int)index ofItem:(id)item
{
	if (item == nil)
	{
		//NSLog(@"Index of nil item: %i", index);
		return [[[myGrammarModel rules] objectAtIndex: index] rootItem];//[MLGrammarRule rootItem];
	}
	else 
	{
		MLRuleOption *ruleOpt = (MLRuleOption *)[item childAtIndex:index];
		MLGrammarRule *new = [[MLGrammarRule alloc] init];
		[new setName: [NSString stringWithFormat: @"%i", [ruleOpt number]]];
		[new setSyntax: [ruleOpt syntax]];
		[new setSemantic: [ruleOpt semantik]];
	//	[new setOptions: [[NSArray alloc] init]];
		return new;		
    }
}

- (BOOL)outlineView:(NSOutlineView *)ov isItemExpandable:(id)item
{
	//if([[item class] isEqual: [MLGrammarRule class]])
		return [item expandable];
	//else
	//	return NO;
}

- (int)outlineView:(NSOutlineView *)ov numberOfChildrenOfItem:(id)item
{
    if (item == nil)
	{
	//	NSLog(@"Item is nil, returned: %i", ruleNameCount);
        return ruleNameCount;
    }
	else
	{
	//	NSLog(@"Asking for child count of item");
		return [item numberOfChildren];	
	}
}

- (id)outlineView:(NSOutlineView *)ov objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	if(item == nil)
		return @"nil";
	else
	{
		if([[tableColumn identifier] isEqualToString: @"name"])
		{
				return [item name];
		}
		if([[tableColumn identifier] isEqualToString: @"syntax"])
		{
			if([item syntax] != nil)
				return [item syntax];
			else
				return @"";
		}
		if([[tableColumn identifier] isEqualToString: @"semantics"])
		{
			if([item semantic] != nil)
				return [item semantic];
			else
				return @"";
		}
		return @"Error";
	}	
}

- (void)dealloc
{
  //  [grammarRule release];
	[grammar release];
    [super dealloc];	
}

@end
