//
//  grammarModel.m
//  bmcc
//
//  Created by Matthias Lange on Mon Apr 19 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "grammarModel.h"
#import "optionModel.h"
#import "bmccMain.h"
#import "MLMorphem.h"
#import "STreeNode.h"


@implementation grammarModel

- (id)init
{
    [super init];
    children = [[NSMutableArray alloc] init];
    data = [[NSMutableDictionary alloc] initWithCapacity: 2];
    nachEpsilon = [[NSMutableDictionary alloc] initWithCapacity: 2];
    firstMengen = [[NSMutableDictionary alloc] initWithCapacity: 2];
    followMengen = [[NSMutableDictionary alloc] initWithCapacity: 2];
    analyseTabelle = [[NSMutableArray alloc] init];
    return self;
}

- (id)initWithParent:(bmccMain *)myParent
{
	[self init];
	parent = myParent;
	return self;
}

- (void)setValue:(NSString *)value forKey:(NSString *)key
{
    [data setValue:value forKey:key];	
}

- (NSString *)keyValue:(NSString *)key
{
    return [data valueForKey:key];	
}

/*- (void)addChild:(grammarModel *)n
{
    [children addObject:n];
}

- (grammarModel *)childAtIndex:(int)i
{
    return [children objectAtIndex:i];
}

- (int)childrenCount
{
    return [children count];	
}*/

- (BOOL)expandable
{
    return ([children count] > 0);	
}

- (NSMutableDictionary *)firstMengen
{
	return firstMengen;
}

- (NSMutableDictionary *)followMengen
{
	return followMengen;
}

- (void)addNTerminal:(NSString *)nterminal
{
	if(!nterminals)
		nterminals = [[NSMutableArray alloc] init];
	[nterminals addObject: nterminal];
}

- (void)addTerminal:(NSString *)terminal
{
	if(!terminals)
		terminals = [[NSMutableArray alloc] init];
	[terminals addObject: terminal];
}

- (void)setLexSpec:(NSString *)spec
{
	if(!lexSpecFile)
		lexSpecFile = [[NSString alloc] init];
	lexSpecFile = spec;
}

-(void)setEpsilonSymbol:(NSString *)eps
{
	epsilon = [[NSString alloc] initWithString: eps];
}

-(NSString *)epsilonSymbol
{
	if(!epsilon)
		epsilon = [[NSString alloc] init];
	return epsilon;
}

-(NSString *)lexSpec
{
	return lexSpecFile;
}

- (NSArray *)nterminals
{
	if(!nterminals)
		nterminals = [[NSMutableArray alloc] init];
	return (NSArray *)nterminals;
}

- (NSArray *)terminals
{
	if(!terminals)
		terminals = [[NSMutableArray alloc] init];
	return (NSArray *)terminals;
}

- (NSArray *)ruleNames
{
	if(!rulesList)
		rulesList = [[NSMutableArray alloc] init];
	return (NSArray *)rulesList;
}

- (NSString *)grammarName
{
	if(!grammarName)
		grammarName = [[NSString alloc] init];
	return grammarName;
}

- (NSString *)startSymbol
{
	if(!startSymbol)
		startSymbol = [[NSString alloc] init];
	return startSymbol;
}

- (NSDictionary *)rules
{
	if(!rules)
		rules = [[NSMutableDictionary alloc] initWithCapacity: 2];
	return (NSDictionary *)rules;
}

- (void)addRuleToList:(NSString *)ruleName
{
	if(!rulesList)
		rulesList = [[NSMutableArray alloc] init];
	[rulesList addObject: ruleName];
}

- (void)addRuleWithName:(NSString *)ruleName withOptions:(NSArray *)options
{
	if(!rules)
		rules = [[NSMutableDictionary alloc] initWithCapacity: 2];
	[rules setValue: options forKey: ruleName];
}

- (void)setGrammarName:(NSString *)name
{
	if(!grammarName)
		grammarName = [[NSString alloc] init];
	grammarName = name;
}

- (void)setStartSymbol:(NSString *)start
{
	if(!startSymbol)
		startSymbol = [[NSString alloc] initWithString: start];
	else
		startSymbol = [NSString stringWithString: start];
}

- (void) fillInNachEpsilon
{
}

- (void) buildNachEpsilon
{
	if([nachEpsilon count]>0)
		return;
	[nachEpsilon setObject: @"1" forKey: epsilon];
	BOOL Done = TRUE, AllEps;
	NSEnumerator *left,*opt,*right;
	id cleft, cright;
	optionModel *copt;
	do
	{
		Done = TRUE;
		left = [rules keyEnumerator];
		while(cleft = [left nextObject])
		{
			//NSLog(@"Cleft: %@", cleft);
			if([nachEpsilon objectForKey:cleft] != nil)
				continue;
			opt = [[rules objectForKey: cleft] objectEnumerator];
			while(copt = [opt nextObject])
			{
				//NSLog(@"Copt: %@", copt);
				if([nachEpsilon objectForKey:cleft] != nil)
					break;
				right = [[copt syntaxList] objectEnumerator];
				AllEps = TRUE;
				while(cright = [right nextObject])
				{
					if([nachEpsilon objectForKey:cright] == nil)
						AllEps = FALSE;
				}
				if(AllEps)
				{
					Done = FALSE;
					[nachEpsilon setObject: @"1" forKey:cleft];
				//	NSLog(@"Nach epsilon: %@",cleft);
				}
			}
		}
	}while(!Done);
}

- (NSArray *) getFirstMenge: (NSString *)S
{
	if([firstMengen objectForKey: S]!=nil)
		return [firstMengen objectForKey: S];
	NSMutableArray *E = [[NSMutableArray alloc] init];
	if([S isEqualToString: epsilon])
	{
		[firstMengen setObject: E forKey: S];
		return E;
	}
	if([terminals containsObject: S])
	{
		[E addObject: S];
		[firstMengen setObject: E forKey: S];
		return E;
	}
	NSEnumerator *optE = [[rules objectForKey: S] objectEnumerator];
	NSEnumerator *rightE;
	id copt,cright;
	while(copt = [optE nextObject])
	{
		rightE = [[copt syntaxList] objectEnumerator];
		while(cright = [rightE nextObject])
		{
			[E addObjectsFromArray: [self getFirstMenge: cright]];
			if(![nachEpsilon objectForKey: cright])
				break;
		}	
	}	
	[firstMengen setObject: E forKey: S];
	return E;
}

- (NSArray *) getFollowMenge: (NSString *)S
{
	if([followMengen objectForKey: S]!=nil)
		return [followMengen objectForKey: S];
	NSMutableArray *E = [[NSMutableArray alloc] init];
	if([S isEqualToString: startSymbol])
	{
		[E addObject: @"_end"];
		[followMengen setObject: E forKey: S];
		return E;
	}
	NSEnumerator *leftE,*optE,*rightE;
	id left,opt,right;
	BOOL Follow = FALSE;
	leftE = [rules keyEnumerator];
	while(left = [leftE nextObject])
	{
		optE = [[rules objectForKey: left] objectEnumerator];
		while(opt = [optE nextObject])
		{
			rightE = [[opt syntaxList] objectEnumerator];
			Follow = FALSE;
			while(right = [rightE nextObject])
			{
				if(Follow)
				{
					[E addObjectsFromArray: [self getFirstMenge: right]];
					if(![nachEpsilon objectForKey: right])
						Follow = FALSE;
				}
				if([right isEqualToString: S])
					Follow = TRUE;	
			}
			if(Follow && ![left isEqualToString:S])
			{
				[E addObjectsFromArray: [self getFollowMenge: left]];
			}
		}
	}
	[followMengen setObject: E forKey: S];
	return E;
}

- (void) buildZielMengen
{
	[self buildNachEpsilon];
	NSEnumerator *leftE,*optE,*rightE;
	id left, right;
	optionModel *opt;
	BOOL AllEps;
	leftE = [rules keyEnumerator];
	while(left = [leftE nextObject])
	{
		//NSLog(@"Left: %@",left);
		optE = [[rules objectForKey: left] objectEnumerator];
		while(opt = [optE nextObject])
		{
			rightE = [[opt syntaxList] objectEnumerator];
			AllEps=TRUE;
			while(right = [rightE nextObject])
			{
				[opt addZielSymbolsFromArray: [self getFirstMenge: right]];
				if(![nachEpsilon objectForKey: right])
				{
					AllEps = NO;
					break;
				}
			}
			if(AllEps)
				[opt addZielSymbolsFromArray: [self getFollowMenge: left]];
		/*	int j, i = [[opt syntaxList] count];
			for(j = 0; j < i; j++)
				NSLog([[opt syntaxList] objectAtIndex: j]);
			NSLog(@"-----------------------");
			i = [[opt zielMenge] count];
			for(j = 0; j < i; j++)
				NSLog([[opt zielMenge] objectAtIndex: j]);
			NSLog(@"=======================");*/
		}
	}
}

//#define _I(a) [NSNumber numberWithInt: a]
//#define _i(a) [a intValue]

- (NSMutableArray *)allOptions
{
	return allOptions;
}


- (void) buildAnalyseTabelle
{
	if([analyseTabelle count]>0)
		return;
	NSMutableArray *Temp;
	NSEnumerator *ntE,*tE;
	BOOL Found;
	id nt,t;
	optionModel *FoundO;
	NSEnumerator *leftE,*optE;
	id left, opt;
	allOptions = [[NSMutableArray alloc] init];
	if([allOptions count]==0)
	{
		leftE = [rules keyEnumerator];
		while(left = [leftE nextObject])
		{
			optE = [[rules objectForKey: left] objectEnumerator];
			while(opt = [optE nextObject])
			{
				//NSLog(@"Opt: %@", [opt syntax]);
				[allOptions addObject: opt];
				
			}
		}
	}
	ntE = [nterminals objectEnumerator];
	while(nt = [ntE nextObject])
	{
		tE = [terminals objectEnumerator];
		Temp = [[NSMutableArray alloc] init];
		while(t = [tE nextObject])
		{
			Found = FALSE;
			optE = [[rules objectForKey: nt] objectEnumerator];
			while(opt = [optE nextObject])
			{
				if([[opt zielMenge] containsObject: t])
				{
					if(Found)
					{
						[self ShowError :[NSString stringWithFormat: @"Die Grammatik ist NICHT LL(1)!"]];
					}
					Found = TRUE;	
					FoundO = opt;
				}
			}
			if(Found)
			{
				//NSLog(@"%@ bei t: %@ nt: %@",[allOptions indexOfObject: opt],t,nt);
				[Temp addObject: [NSNumber numberWithInt: [allOptions indexOfObject: FoundO]]];
			}
			else
			{				
				[Temp addObject: [NSNumber numberWithInt: -1]];
			}
		}
		[analyseTabelle addObject: Temp];
	}
	
}

- (void) ShowError: (NSString *)error
{
	[parent showErrorDrawer];
	[NSThread detachNewThreadSelector: @selector(showErrors:) toTarget: parent withObject: error];
}

- (NSMutableArray *)analyzeMorpheme: (NSMutableArray *)morpheme
{	
	performStepOne = NO;
	performStepOne = NO;
	[self buildAnalyseTabelle];

	NSMutableArray *ResultTree = [[NSMutableArray alloc] init];
	NSMutableArray *Stack = [[NSMutableArray alloc] init];
	int MorphemPos = 0, StackPos = -1;
	BOOL HadError = NO;
	[Stack addObject: [self startSymbol]];
	id StackTop;
	MLMorphem *MorphemCurr;
	optionModel *OptCurr;
	StackPos++;
	while(StackPos>=0)
	{
		StackTop = [Stack objectAtIndex: StackPos];
		//NSLog(@"Stack top: %@", StackTop);
		if(MorphemPos>=[morpheme count])
		{
			[self ShowError: [NSString stringWithFormat: @"Ende der Datei nicht erwartet! %@ erwartet.",StackTop]];
			return nil;
		}
		MorphemCurr = (MLMorphem *)[morpheme objectAtIndex: MorphemPos];
		if([[self terminals] containsObject: StackTop])
		{
			[ResultTree addObject: [NSNumber numberWithInt: -MorphemPos-1]];
			if(![StackTop isEqualToString: [MorphemCurr syntax]])
			{
				[self ShowError: [NSString stringWithFormat: @"Fehler bei der syntaktischen Analyse in Zeile %i:%i %@ erwartet, %@ gefunden.",[MorphemCurr line], [MorphemCurr column], StackTop, [MorphemCurr syntax]]];
				HadError = YES;
			}
			else
				MorphemPos++;
			[Stack removeObjectAtIndex: StackPos];
			StackPos--;
			continue;
		}
		if([[self nterminals] containsObject: StackTop])
		{
			int t = [[self terminals] indexOfObject: [MorphemCurr syntax]];
			int nt = [[self nterminals] indexOfObject: StackTop];
			//NSLog(@"t: %i, nt: %i", t, nt);
			if(nt==NSNotFound)
			{
				[self ShowError :[NSString stringWithFormat: @"Interner Fehler der Syntaxanalyse..."]];
				return nil;
			}
			if(t==NSNotFound)
			{
				[self ShowError: [NSString stringWithFormat: @"Interner Fehler der Syntaxanalyse... : %@ ist nicht als Terminal definiert!",[MorphemCurr syntax]]];
				return nil;
			}
			int opt = [[[analyseTabelle objectAtIndex: nt] objectAtIndex: t] intValue];
		//	NSLog(@"Opt: %i", opt);
			if(opt<0 || opt>= [allOptions count])
			{
				[self ShowError: [NSString stringWithFormat: @"Syntaxfehler vor Zeile %i:%i %@!",[MorphemCurr line], [MorphemCurr column], [MorphemCurr syntax]]];
				return nil;
			}
			[ResultTree addObject: [NSNumber numberWithInt: opt]];
			OptCurr = [allOptions objectAtIndex: opt];
			[Stack removeObjectAtIndex: StackPos];
			StackPos--;
			int i,l;
			l = [[OptCurr syntaxList] count];
			for(i=l-1;i>=0;i--)
			{
				if(![[[OptCurr syntaxList] objectAtIndex: i] isEqualToString: epsilon])
				{
					[Stack addObject: [[OptCurr syntaxList] objectAtIndex: i]];
					StackPos++;
				}
			}
			continue;
		}
		[self ShowError: [NSString stringWithFormat: @"StackTop %@ not understood!",StackTop]];
		return nil;
	}
	return ResultTree;
}

typedef struct
{
	NSMutableArray *semantik;
	int syntaxLen;
}OutputHelper;

/// ersetzt 'Tag' in 'Outer' durch 'Inner'
- (void)InsertArray:(NSMutableArray *)Outer withTag: (NSString *)Tag withInner: (NSMutableArray *)Inner
{
	int i,j;
	for(i=0;i<[Outer count];i++)
	{
		if([Tag isEqualToString: [Outer objectAtIndex: i]])
		{
			[Outer removeObjectAtIndex: i];
			for(j=0;j<[Inner count];j++)
			{
				[Outer insertObject: [Inner objectAtIndex: j] atIndex: i];
				i++;
			}	
		}
	}
}

int GlobalCounter = 1;

///    macht aus @-strings eindeutige namen durch anhngen einer nummer
///    %SetJumpMark(@WhileHead)  Condition  JZ @WhileEnd  Statement  JMP @WhileHead  %SetJumpMark(@WhileEnd)
/// -> %SetJumpMark(WhileHead1)  Condition  JZ WhileEnd2  Statement  JMP WhileHead1  %SetJumpMark(WhileEnd2)
- (void) MakeEdsUnique:(NSMutableArray *)A
{
	NSMutableDictionary *D = [[NSMutableDictionary alloc] init];
	NSMutableString *MS, *AMS;
	int a,i,j;
	for(a=0;a<[A count];a++)
	{
		AMS = [A objectAtIndex: a];
		for(i=0;i<[AMS length]-1;i++)
		{
			if([AMS characterAtIndex: i]!='@')
				continue;
			if([AMS characterAtIndex: i+1]=='@')
				continue;
			MS = [[NSMutableString alloc] init];
			i++;
			j = i;
			while(i<[AMS length] && [AMS characterAtIndex: i]!='@' && [AMS characterAtIndex: i]!=' ' && [AMS characterAtIndex: i]!=')')
				i++;
			[MS appendString: [AMS substringWithRange: NSMakeRange(j,i-j)]];
			[D setObject: [NSNumber numberWithInt: GlobalCounter++] forKey: MS];
			i--;
		}
	}
	NSEnumerator *kE = [D keyEnumerator];
	
	while(MS = [kE nextObject])
	{
		for(a=0;a<[A count];a++)
		{
			AMS = [A objectAtIndex: a];
			[AMS replaceOccurrencesOfString: [NSString stringWithFormat:@"@%@",MS]
			                     withString: [NSString stringWithFormat:@"%@%i",MS,[[D objectForKey: MS] intValue]]
			                        options: NSLiteralSearch 
			                          range: NSMakeRange(0,[AMS length])];
		}
	}
}

- (OutputHelper) recursiveTile: (NSMutableArray *)syntaxTree Morpheme: (NSMutableArray *)morpheme Pos: (int)pos
{
	OutputHelper OH,OHIn;
	NSMutableArray *replacements = [[NSMutableArray alloc] init];
	optionModel *OptCurr = [allOptions objectAtIndex: [[syntaxTree objectAtIndex: pos] intValue] ];
	int subCount = [[OptCurr syntaxList] count], count=0, i, j;
	OH.semantik = [[NSMutableArray alloc] init];
	NSMutableString *MS;
	
	for(i=0;i<[[OptCurr semantikList] count];i++)
	{
		MS = [[NSMutableString alloc] init];
		[MS appendString: [[OptCurr semantikList] objectAtIndex: i]];
		[OH.semantik addObject: MS];
	}
//	[OH.semantik addObjectsFromArray: [OptCurr semantikList]];
	OH.syntaxLen = 1;
	
	[self MakeEdsUnique: OH.semantik];
	
	if([[self epsilonSymbol] isEqualToString: [[OptCurr syntaxList] objectAtIndex: 0]])
	{
		return OH;
	}
	pos++;
	do
	{
		count ++;
		if([[syntaxTree objectAtIndex: pos] intValue]>=0)
		{
			OptCurr = [allOptions objectAtIndex: [[syntaxTree objectAtIndex: pos] intValue] ];
			OHIn = [self recursiveTile: syntaxTree Morpheme: morpheme Pos: pos];
			[self InsertArray: OH.semantik withTag: [OptCurr leftSide] withInner: OHIn.semantik];
			pos          += OHIn.syntaxLen;
			OH.syntaxLen += OHIn.syntaxLen;
			[replacements addObject: @""];
			continue;
		}	
		[replacements addObject: [[morpheme objectAtIndex: (-[[syntaxTree objectAtIndex: pos] intValue]-1)] semantic]];
		pos ++;
		OH.syntaxLen ++;
	}while(count < subCount);
	
	for(i=0;i<[OH.semantik count];i++)
	{
		MS = [OH.semantik objectAtIndex: i];
		for(j=0;j<[replacements count];j++)
		{
			[MS replaceOccurrencesOfString: [NSString stringWithFormat: @"{%i}",j] 
			                    withString: [replacements objectAtIndex: j] 
			                       options: NSLiteralSearch 
			                         range: NSMakeRange(0,[MS length])];
		}
//		[OH.semantik replaceObjectAtIndex: i withObject: MS];
	}
	
	return OH;
}

- (NSMutableArray *) callSemantikFkt: (NSString *)name withParams: (NSArray *)params inLine: (int)Line inStep: (int)Step
{
	NSMutableArray *Erg = [[NSMutableArray alloc] init];
	if([name isEqualToString: @"SetJumpMark"])
	{
		if(Step==1)
			return nil;
		[jmpPositions setObject: [NSNumber numberWithInt: Line] forKey: [params objectAtIndex: 0]];
		return Erg;
	}
	if([name isEqualToString: @"EnterLevel"])
	{
		if(Step==1)
		{
			STreeNode *NewNode = [mainNode createChildNode: (NSString *)[params objectAtIndex: 0]];
			if(NewNode==nil)
			{
				[self ShowError: [NSString stringWithFormat:@"Funktion %@ zweimal definiert!",[params objectAtIndex: 0]]];
				return Erg;
			}
			mainNode = NewNode;
		//	NSLog(@"----EnterLevel: Current=%@",[mainNode name]);
			return nil;
		}
		if(Step>1)
		{
			STreeNode *NewNode = [mainNode getChildNode: [params objectAtIndex: 0]];
			if(NewNode==nil)
			{
				[self ShowError: [NSString stringWithFormat:@"Funktion %@ wurde nicht gefunden (interner Fehler)!",[params objectAtIndex: 0]]];
				return Erg;
			}
			[NewNode setLineNumber: Line];
			mainNode = NewNode;
			if(Step<3)
				return nil;
			return Erg;
		}
	}
	if([name isEqualToString: @"LeaveLevel"])
	{
		if([mainNode parent]==nil)
		{
			[self ShowError: [NSString stringWithFormat:@"LeaveLevel zuviel!"]];
			return nil;
		}
		mainNode = [mainNode parent];
		if(Step<3)
			return nil;
		if(Step==3)
			return Erg;
	}	
	if([name isEqualToString: @"MakeAddress"])
	{
		if(Step==1)
		{
			if([memDef objectForKey: [params objectAtIndex: 0]]==nil)
			{
				[memDef setObject: [NSNumber numberWithInt: memDefCounter++] forKey: [params objectAtIndex: 0]];
			}
			return Erg;
		}
	}	
	if([name isEqualToString: @"Define"])
	{
		if(Step==1)
		{
			if([memDef objectForKey: [params objectAtIndex: 0]]==nil)
			{
				[memDef setObject: [NSNumber numberWithInt: memDefCounter++] forKey: [params objectAtIndex: 0]];
			}
			BOOL e = [mainNode defineName: [params objectAtIndex: 0] withType: [params objectAtIndex: 1]];
			if(!e)
			{
				[self ShowError: [NSString stringWithFormat:@"Variable %@ wurde zweimal definiert!",[params objectAtIndex: 0]]];
				return Erg;
			}
			return Erg;
		}
	}	
	if([name isEqualToString: @"CheckDefine"])
	{
		if(Step==1)
		{
			return nil;
		}
		if(Step==2)
		{
			BOOL e = [mainNode isNameDefined: [params objectAtIndex: 0] withTypes: [params objectAtIndex: 1]];
			if(!e)
			{
				[self ShowError: [NSString stringWithFormat:@"Variable %@ wurde nicht definiert!",[params objectAtIndex: 0]]];
				return Erg;
			}
			return Erg;
		}
	}	
	if([name isEqualToString: @"All"])
	{
		if(Step==1)
			return nil;
		if(Step==2)
		{
			NSArray *types = [[params objectAtIndex: 1] componentsSeparatedByString: @":"];
			NSMutableArray *Defs = [[NSMutableArray alloc] init];
			int i;
			for(i=0;i<[types count];i++)
				[Defs addObjectsFromArray: [mainNode getDefinitionsWithType: [types objectAtIndex: i]]];
			NSEnumerator *dE = [Defs objectEnumerator];
			NSString *Def;
			NSMutableString *MS;
			while(Def = [dE nextObject])
			{
				MS = [[NSMutableString alloc] init];
				[MS appendString: [params objectAtIndex: 0]];
				[MS replaceOccurrencesOfString: @"{Def}"
				                    withString: Def
				                       options: NSLiteralSearch 
				                         range: NSMakeRange(0,[MS length])];
				[Erg addObjectsFromArray: [MS componentsSeparatedByString: @":"]];
			}
			return Erg;
		}
	}
	if([name isEqualToString: @"AllR"])
	{
		if(Step==1)
			return nil;
		if(Step==2)
		{
			NSArray *types = [[params objectAtIndex: 1] componentsSeparatedByString: @":"];
			NSMutableArray *Defs = [[NSMutableArray alloc] init];
			int i;
			for(i=0;i<[types count];i++)
				[Defs addObjectsFromArray: [mainNode getDefinitionsWithType: [types objectAtIndex: i]]];
			NSEnumerator *dE = [Defs reverseObjectEnumerator];
			NSString *Def;
			NSMutableString *MS;
			while(Def = [dE nextObject])
			{
				MS = [[NSMutableString alloc] init];
				[MS appendString: [params objectAtIndex: 0]];
				[MS replaceOccurrencesOfString: @"{Def}"
				                    withString: Def
				                       options: NSLiteralSearch 
				                         range: NSMakeRange(0,[MS length])];
				[Erg addObjectsFromArray: [MS componentsSeparatedByString: @":"]];
			}
			return Erg;
		}
	}
	if([name isEqualToString: @"ResolveMarkToLine"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			[Erg addObject: [NSString stringWithFormat: @"%i",[[jmpPositions objectForKey: [params objectAtIndex:0]] intValue]]];
		}
		return Erg;
	}
	
	if([name isEqualToString: @"ResolveLevelToLine"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			[Erg addObject: [NSString stringWithFormat: @"%i", [[mainNode findNode: [params objectAtIndex:0]] lineNumber]]];
		}
		return Erg;
	}
	if([name isEqualToString: @"ResolveLevelToAddress"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			STreeNode *N = [mainNode findNode: [params objectAtIndex:0]];
			NSMutableString *MS = [[NSMutableString alloc] init];
			[MS appendString: @"PL0_"];
			[MS appendString: [N name]];
			while([N parent]!=nil)
			{
				[MS appendString: [params objectAtIndex: 1]];
				N = [N parent];
				[MS appendString: [N name]];
			}
			[Erg addObject: [NSString stringWithFormat: @"%i",[[memDef objectForKey: MS] intValue]]];
		}
		return Erg;
	}
	if([name isEqualToString: @"ResolveCurrentLevelToAddress"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			STreeNode *N = mainNode;
			NSMutableString *MS = [[NSMutableString alloc] init];
			[MS appendString: @"PL0_"];
			[MS appendString: [N name]];
			while([N parent]!=nil)
			{
				[MS appendString: [params objectAtIndex: 0]];
				N = [N parent];
				[MS appendString: [N name]];
			}
			[Erg addObject: [NSString stringWithFormat: @"%i",[[memDef objectForKey: MS] intValue]]];
		}
		return Erg;
	}
	if([name isEqualToString: @"ResolveLevelToString"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			STreeNode *N = [mainNode findNode: [params objectAtIndex:0]];
			NSMutableString *MS = [[NSMutableString alloc] init];
			[MS appendString: @"PL0_"];
			[MS appendString: [N name]];
			while([N parent]!=nil)
			{
				[MS appendString: [params objectAtIndex: 1]];
				N = [N parent];
				[MS appendString: [N name]];
			}
			[Erg addObject: MS];
		}
		return Erg;
	}
	if([name isEqualToString: @"ResolveCurrentLevelToString"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			STreeNode *N = mainNode;
			NSMutableString *MS = [[NSMutableString alloc] init];
			[MS appendString: @"PL0_"];
			[MS appendString: [N name]];
			while([N parent]!=nil)
			{
				[MS appendString: [params objectAtIndex: 0]];
				N = [N parent];
				[MS appendString: [N name]];
			}
			[Erg addObject: MS];
		}
		return Erg;
	}
	if([name isEqualToString: @"MakeAddressForCurrentLevel"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			STreeNode *N = mainNode;
			NSMutableString *MS = [[NSMutableString alloc] init];
			[MS appendString: @"PL0_"];
			[MS appendString: [N name]];
			while([N parent]!=nil)
			{
				[MS appendString: [params objectAtIndex: 0]];
				N = [N parent];
				[MS appendString: [N name]];
			}
			if([memDef objectForKey: MS]==nil)
			{
				[memDef setObject: [NSNumber numberWithInt: memDefCounter++] forKey: MS];
			}
		}
		return Erg;
	}
	if([name isEqualToString: @"ResolveAddress"])
	{
		if(Step<3)
			return nil;
		if(Step==3)
		{
			[Erg addObject: [NSString stringWithFormat: @"%i",[[memDef objectForKey: [params objectAtIndex:0]] intValue]]];
		}
		return Erg;
	}
	[self ShowError: [NSString stringWithFormat: @"Funktion %@ nicht verstanden (interner Fehler)!",name]];
	return Erg;
}

- (void) performStep: (NSMutableArray *)currentSemantik withNumber: (int) N
{
	int i, j, bracketPos,CurrentLine = 0;
	NSString *cur, *name, *paramsStr;
	NSArray *params;
	NSMutableArray *Erg;
	for(i=0;i<[currentSemantik count];i++)
	{
		cur = [currentSemantik objectAtIndex: i];
	//	NSLog(@"i: %i cur: %@",i,cur);
		if([cur characterAtIndex: 0]!='_')
		{
			if([cur isEqualToString: @"$"])
				CurrentLine++;
			continue;
		}
		bracketPos = 1;
		while(bracketPos < [cur length] && [cur characterAtIndex: bracketPos]!='(')
			bracketPos++;
		if(bracketPos>=[cur length])
			continue;
		name = [cur substringWithRange: NSMakeRange(1,bracketPos-1)];
		paramsStr = [cur substringWithRange: NSMakeRange(bracketPos+1,[cur length]-bracketPos-2)];
		params = [paramsStr componentsSeparatedByString: @","];
		Erg = [self callSemantikFkt: name withParams: params inLine: CurrentLine inStep: N];		
		if(Erg == nil)
			continue;
		[currentSemantik removeObjectAtIndex: i];
		for(j=0;j<[Erg count];j++)
		{
			[currentSemantik insertObject: [Erg objectAtIndex: j] atIndex: i];
			if([[Erg objectAtIndex: j] isEqualToString: @"$"])
				CurrentLine++;
			i++;
		}
		i--;
	}
}

- (NSMutableArray *) buildSemantikWithSyntax: (NSMutableArray *)syntaxTree andMorphemen: (NSMutableArray *)morpheme withPass: (int)pass
{
	NSMutableArray *Semantik;
	OutputHelper OH = [self recursiveTile: syntaxTree Morpheme: morpheme Pos: 0];
	Semantik = [[NSMutableArray alloc] initWithArray: OH.semantik];
	
	mainNode = [[STreeNode alloc] initWithParent: nil andName: @"Main"];
	jmpPositions = [[NSMutableDictionary alloc] init];
	memDef = [[NSMutableDictionary alloc] init];
	memDefCounter=0;
	
	if(pass == 1)
	{
		#ifdef _DEBUGGRAMMARMODEL
			NSLog(@"Perform pass 1");
		#endif
		[self performStep: Semantik withNumber: 1];
	}
	else if(pass == 2)
	{
		#ifdef _DEBUGGRAMMARMODEL
			NSLog(@"Perform pass 2");
		#endif
		[self performStep: Semantik withNumber: 1];
		[self performStep: Semantik withNumber: 2];
		//performStepTwo = YES;
	}
	else if(pass == 3)
	{
		#ifdef _DEBUGGRAMMARMODEL
			NSLog(@"Perform pass 3");
		#endif
		[self performStep: Semantik withNumber: 1];
		[self performStep: Semantik withNumber: 2];
		[self performStep: Semantik withNumber: 3];
	}
	return Semantik;
}

#define GetDic(A,B) [A objectForKey: B]
#define SetDic(A,B,C) [A setObject: C forKey: B]
#define GetDicI(A,B) [[A objectForKey: B] intValue]
#define SetDicI(A,B,C) [A setObject: [NSNumber numberWithInt: C] forKey: B]
#define IncDicI(A,B) SetDicI(A,B,GetDicI(A,B)+1)
/*

- (void) executeProgram: (NSMutableArray *)program
{
	NSMutableDictionary *Data = [[NSMutableDictionary alloc] init];
	SetDicI(Data,@"_PC",0);
	while(GetDicI(Data,@"_PC")<=[program count])
	{
		if([[program objectAtIndex: GetDicI(Data,@"_PC")] isEqualToString: @""])
		{
			IncDicI(Data,@"_PC");
			continue;
		}
		
		
	}
}
*/

- (NSArray *)analyseTable
{
	return [analyseTabelle retain];
}


- (void)dealloc
{
    [children release];
    [data release];
	[nterminals release];
	[terminals release];
	[rulesList release];
	[rules release];
	[lexSpecFile release];
	[epsilon release];
	[nachEpsilon release];
	[firstMengen release];
	[followMengen release];
	[grammarName release];
	[startSymbol release];
	[analyseTabelle release];
    [super dealloc];	
}

@end
